#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <time.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#include "client_socket.h"
#include "network.h"

ClientSocket::ClientSocket(const char* host, uint16_t port) {
	
	const SSL_METHOD *method;
	method = SSLv2_client_method();
	if(!method) {
		fprintf(stderr, "error creating method\n");
		ERR_print_errors_fp(stderr);
		throw 0;
	}
	
	ctx = SSL_CTX_new((SSL_METHOD *)method);
	if(!ctx) {
		fprintf(stderr, "error creating ctx\n");
		ERR_print_errors_fp(stderr);
		throw 0;
	}
	
	struct sockaddr_in sa;
	
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if(sock==-1) {
		perror("socket");
		throw 0;
	}
	
	memset(&sa, 0, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sa.sin_addr.s_addr = inet_addr(host);
	
	if(connect(sock, (struct sockaddr*)&sa, sizeof(sa))==-1) {
		perror("connect");
		throw 0;
	}
	
	ssl = SSL_new(ctx);
	SSL_set_fd(ssl, sock);
	if(SSL_connect(ssl)<0) {
		fprintf(stderr, "error connectiong through ssl\n");
		ERR_print_errors_fp(stderr);
		throw 0;
	}
	
	br.ssl = ssl;
	bs.ssl = ssl;
}

void ClientSocket::process_packet(uint8_t* data, uint16_t size) {
	uint8_t type = *data;
	if(type!=1) return;
	
#ifdef DEBUG
	printf("got: type=%u, size=%u\n", type, size);
#endif
	
	uint8_t* p = data+5;
	
	net->write(p, size);
}

void ClientSocket::loop() {
	try {
		br.recv();
	} catch(int e) {
		if(e==0) {
			fprintf(stderr, "read error\n");
		}
		if(e==1) {
			fprintf(stderr, "connection closed\n");
		}
		close(true);
		throw 0;
	}
	
#ifdef DEBUG
	printf("loop data_size=%u\n", br.data_size);
#endif
	
	if(br.data_size<5) return;
	
	uint8_t* p = br.data;
	
	uint32_t size = *(uint32_t*)(p+1);
	if(br.data_size<(size+5)) return;
	
	process_packet(br.data, size);
	
	br.truncate(size+5);
}

void ClientSocket::close(bool closed) {
	if(!closed) {
		SSL_shutdown(ssl);
	}
	SSL_free(ssl);
	shutdown(sock, SHUT_RDWR);
	::close(sock);
}

void ClientSocket::write(uint8_t* data, size_t size) {
	try {
		bs.write(data, size);
		bs.send();
	} catch(int e) {
		printf("closed2: %d\n", e);
		close(true);
	}
}

