#include <stdio.h>
#include <sys/socket.h>
#include <errno.h>
#include <signal.h>
#include <openssl/ssl.h>

#include "client_socket.h"
#include "network.h"

bool end;

void sig_handler(int signo)
{
	end = true;
}

void loop(ClientSocket* cs, Network* net) {
	fd_set fds;
	FD_ZERO(&fds);
	
	FD_SET(cs->sock, &fds);
	FD_SET(net->h, &fds);
	int max = cs->sock;
	if((net->h)>max) max = (net->h);
	
	struct timeval tv;
	tv.tv_sec = 0;
	tv.tv_usec = 500;
	int res = select(max+1, &fds, NULL, NULL, &tv);
	if(res<0) {
		if(errno==EINTR) return;
		perror("select");
		return;
	}
	if(res==0) return;
	if(FD_ISSET(cs->sock, &fds)) cs->loop();
	if(FD_ISSET(net->h, &fds)) net->loop();
}

int main(int argc, char* argv[]) {
	const char* host = "127.0.0.1";
	uint16_t port = 1234;
	int ifn = 0;
	
	if(argc>1) {
		int i = 1;
		while(i<argc) {
			if(strcmp(argv[i],"-n")==0) {
				if(i<(argc-1)) {
					ifn = atoi(argv[i+1]);
					i+=2;
					continue;
				}
			}
			if(strcmp(argv[i],"-h")==0) {
				if(i<(argc-1)) {
					host = argv[i+1];
					i+=2;
					continue;
				}
			}
			if(strcmp(argv[i],"-p")==0) {
				if(i<(argc-1)) {
					port = (uint16_t)atoi(argv[i+1]);
					i+=2;
					continue;
				}
			}
			if(strcmp(argv[i],"--help")==0) {
				printf("usage: client [-h host] [-p port] [-n tun_no]\n");
				return 0;
			}
			i++;
		}
	}
	
	printf("host=%s, port=%u, device=%d\n", host, port, ifn);
	
	srand(time(NULL));
	
	SSL_library_init();
	OpenSSL_add_all_algorithms();
	SSL_load_error_strings();
	
	ClientSocket cs(host, port);
	Network net(ifn);
	
	cs.net = &net;
	net.cs = &cs;
	
	end = false;
	signal(SIGINT, sig_handler);
	
	while(!end) {
		try {
			loop(&cs, &net);
		} catch(...) {
			net.close();
			return 0;
		}
	}
	
	cs.close(false);
	net.close();

	return 0;
}


