#include <stdlib.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <arpa/inet.h> 
#include <sys/select.h>
#include <sys/time.h>
#include <errno.h>
#include <stdarg.h>

#ifdef LINUX
#include <linux/if.h>
#include <linux/if_tun.h>
#endif

#ifdef FREEBSD
#include <net/if.h>
#include <net/if_tun.h>
#endif

#include "network.h"
#include "client_socket.h"

// BUF_SIZE > MTU
#define BUF_SIZE 10000

Network::Network(int n) {
	for(int i=0;i<6;i++) mac[i] = (uint8_t)rand()%256;
	printf("my mac=%02x:%02x:%02x:%02x:%02x:%02x\n", mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	
	char if_name[IFNAMSIZ];
	sprintf(if_name, "tap%d", n);
	printf("trying to get %s\n", if_name);
	
#ifdef LINUX
	int flags = IFF_TAP | IFF_NO_PI;
	if(n==-1) if_name[0] = 0;
	
	if( (h = open("/dev/net/tun" , O_RDWR)) < 0 ) {
		perror("Opening /dev/net/tun");
		throw 0;
	}
	
	struct ifreq ifr;
	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_flags = flags;
	strncpy(ifr.ifr_name, if_name, IFNAMSIZ);
	
	if( (ioctl(h, TUNSETIFF, (void *)&ifr)) < 0 ) {
		perror("ioctl(TUNSETIFF)");
		::close(h);
		throw 0;
	}
	
	strcpy(if_name, ifr.ifr_name);
#else
#ifdef FREEBSD
	char device[100];
	sprintf(device, "/dev/%s", if_name);
	if( (h = open(device , O_RDWR)) < 0 ) {
		perror("Opening device");
		throw 0;
	}
#else
#error "choose OS"
#endif
#endif
	printf("if_name=%s\n", if_name);
	
	printf("network opened\n");
}

void Network::loop() {
#ifdef DEBUG
	printf("network::loop got something\n");
#endif
	
	uint8_t buf0[BUF_SIZE];
	int len = read(h, buf0, sizeof(buf0));
	if(len>0) {
		uint8_t buf[len+5];
		
		buf[0] = 1;
		memcpy(buf+1, &len, sizeof(len));
		memcpy(buf+5, buf0, len);
		cs->write(buf, len+5);
	} else {
		printf("network::loop read error\n");
	}
}

void Network::close() {
	printf("network closed\n");
	//close(h);
}

void Network::write(uint8_t* data, size_t size) {
#ifdef DEBUG
	printf("network::write size=%u\n", size);
#endif
	::write(h, data, size);
}
